<?php
/* Convert each line of a file to an array

Opens a file and explode each line of the file by an optional 
separation character, if the function is called with only the
filepath parameter, the "|" character will be used.

PARAMETERS:
$sFile: filepath to datafile 
$sSeparate: the character words are separated by on each line(OPTIONAL)

RETURNS:
ARRAY: file data in multi-dimensional array (eg [0][1]: second element on first line)
FALSE: an error has occured

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 25-01-2024
*/

function RawToArray($sFile,$sSeparate="|"){

	global $_CONTEXT;

    if(!file_exists($sFile)){
		$_CONTEXT['errors'][] = array('error'=>'File <b>'.htmlspecialchars($sFile).'</b> doesn\'t exist','type'=>'1','fatal'=>'1','line'=>__LINE__,'file'=>__FILE__);
		return(false);
	}
    if(!filesize($sFile)){
		// if the file is empty, fatal will not be set to 1
		$_CONTEXT['errors'][] = array('error'=>'File <b>'.htmlspecialchars($sFile).'</b> is empty','type'=>'1','fatal'=>'0','line'=>__LINE__,'file'=>__FILE__);
		return(false);
	}
    if(!$aFileData = @file($sFile)){
		$_CONTEXT['errors'][] = array('error'=>'Failed to open location <b>'.htmlspecialchars($sFile).'</b>','type'=>'1','fatal'=>'1','line'=>__LINE__,'file'=>__FILE__);
		return(false);
	}
	$aDataBuffer = array();
	for($x=0;$x<count($aFileData);$x++){
		if(!$aDataBuffer[$x] = @explode($sSeparate,$aFileData[$x])){
			if($aFileData[$x]==""){
				continue;
			}
		}
	}
    return($aDataBuffer);
}
?>