<?php
/* Logs an error if logging is enabled

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 14-02-2024
*/
if(!defined('IN_SCRIPT')){
	exit;
}
// if there's nothing to do, then we are leaving again 
if(isset($_CONTEXT['errors'][0]) && (isset($_CONTEXT['log']) && $_CONTEXT['log'])){
	// let's see if the default log is set
	if(!isset($_PATHS['write_dest'])){
		// your configuration is screwed
		$sTempDest = $_PATHS['log_root']."/thc.txt";
		if(file_exists($sTempDest)){
			// it's bad but at least we can continue
			$_PATHS['write_dest'] = $sTempDest;
			$_CONTEXT['errors'][] = array('error'=>'\$_PATHS[\'write_dest\'] not set in vars.php','type'=>'2','fatal'=>'0','json'=>'0','line'=>__LINE__,'file'=>__FILE__);
		}
		else{
			// can't log anything
			die("Can only write logs if <b>\$_PATHS['write_dest']</b> has been set, no such var in vars.php");
		}
	}
	// if the writing function wasn't included yet then try to do it now
	if(!function_exists("WriteF")){
		$sDest = $_PATHS['functions_root']."/fwrite.php";
		include_once($sDest);
	}
	if(!isset($_CONTEXT['l_formats'])){
		include($_PATHS['data_root']."/logsdata.php");
	}
	if(!isset($_CONTEXT['l_formats'][0]['file'])){
		// log format data file is screwed
		die("No log properties configured in ".$_PATHS['data_root']."/logsdata.php");
	}
	// create the write buffer
	$sWrite = "";
	$iErrors = count($_CONTEXT['errors']);
	$aLogFile = explode("/",$_PATHS['write_dest']);
	$sMainLogFile = $aLogFile[count($aLogFile)-1];
	$sLastLogFile = $sMainLogFile;
	$iLogKey = -1;
	// whenever we encounter a false configured log, the array indice will be overwritten (and hopefully corrected) then will be processed again
	$bPreventLoop = false;
	for($x=0;$x<$iErrors;$x++){
		if(isset($_CONTEXT['errors'][$x]['log'])){
			// switch to a different logbook
			if($sLastLogFile!=$_CONTEXT['errors'][$x]['log']){
				// write first part of the buffer to previous log and reset buffer
				if($sWrite!=""){
					// no log no problem...sort of
					WriteF($_PATHS['log_root']."/".$sLastLogFile,$sWrite,(!file_exists($_PATHS['write_dest']) ? "w" : "a"));
					$sWrite = "";
				}
				// let's see if this log is configured
				$bFoundLog = false;
				for($y=0;$y<count($_CONTEXT['l_formats']);$y++){
					if($_CONTEXT['l_formats'][$y]['file']==$_CONTEXT['errors'][$x]['log']){
						$sLastLogFile = $_CONTEXT['l_formats'][$y]['file'];
						$iLogKey = $y;
						$_CONTEXT['writelogs'][$iLogKey] = array();
						$bFoundLog = true;
						break;
					}
				}
				if(!$bFoundLog){
					// no such log found, log this error in the default log as important and quit
					$sError = "Invalid call to logfile that is not configured in Data/logsdata.php";
					$sWrite .= $_SERVER['REMOTE_ADDR']."|".time()."|".$sError."|".__FILE__."|".__LINE__."|2|1\n";
					WriteF($_PATHS['write_dest'],$sWrite,(!file_exists($_PATHS['write_dest']) ? "w" : "a"));
					if(isset($_CONTEXT['errors'][$x]['json']) && $_CONTEXT['errors'][$x]['json']==1){
						// handle json
						if(isset($aDataR)){
							$aDataR['jmessage'] = $sError;
						}
						else{
							$aDataR = array();
							// implement compatibility with other variables holding the error message
							if(isset($_CONTEXT['errors'][$x]['jsonvar'])){
								$aDataR[$_CONTEXT['errors'][$x]['jsonvar']] = $sError;
							}
							else{
								$aDataR['jmessage'] = $sError;
							}
							$aDataR['jresult'] = false;
						}
						die(json_encode($aDataR));
					}
					if(isset($_CONTEXT['errors'][$x]['screen']) && $_CONTEXT['errors'][$x]['screen']==1){
						// handle screen.php files
						die("<b id=h7>".$sError."</b>");
					}
					else{
						// handle error as we normally would
						die($sError);
					}
				}
			}
		}
		else{
			// no log set, so switch back to default log
			if($sWrite!=""){
				WriteF($_PATHS['log_root']."/".$sLastLogFile,$sWrite,(!file_exists($_PATHS['log_root']."/".$sLastLogFile) ? "w" : "a"));
				$sWrite = "";
			}
			$sLastLogFile = $sMainLogFile;
			$iLogKey = -1;
		}
		// check if the error properties are properly set
		$bConfiguredProperly = true;
		// if the error isn't properly set we can just discard it
		$sTemp = "";
		if($iLogKey==-1){
			// log hasn't been set, make sure indice 3 matches the main log
			if($_CONTEXT['l_formats'][3]['file']==$sMainLogFile){
				$iLogKey = 3;
			}
			else{
				// this is bad, let's make sure it's at least defined in the file logsdata.php
				for($z=0;$z<count($_CONTEXT['l_formats']);$z++){
					if($_CONTEXT['l_formats'][$z]['file']==$sMainLogFile){
						$iLogKey = $z;
						break;
					}
				}
				if($iLogKey==-1){
					// didn't find it, we're screwed
					die("Main log hasn't been configured, either turn logs off or properly configure ".$_PATHS['data_dir']."/logsdata.php");
				}
			}
		}
		for($z=0;$z<count($_CONTEXT['l_formats'][$iLogKey]['format']['labels']);$z++){
			if($z!=0){
				$sTemp .= "|";
			}
			if(!isset($_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]])){
				// exceptions that are optional
				if($_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]=="ip"){
					$_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]] = $_SERVER['REMOTE_ADDR'];
				}
				elseif($_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]=="time"){
					$_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]] = time();
				}
				elseif($_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]=="timestamp"){
					$_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]] = time();
				}
				elseif($_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]=="type"){
					// if not set to 2, it's an error that doesn't require attention
					$_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]] = 0;
				}
				elseif($_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]=="fatal"){
					// not set, so assume it's allowed to continue parsing
					$_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]] = 0;
				}
				else{
					// not properly configured just abort
					$bConfiguredProperly = false;
					break;
				}
			}
			$sTemp .= $_CONTEXT['errors'][$x][$_CONTEXT['l_formats'][$iLogKey]['format']['labels'][$z]];
		}
		if($bConfiguredProperly){
			// add to write buffer
			$sWrite .= $sTemp."\n";
		}
		else{
			// error is not properly configured, let's overwrite the error and try again while making sure we don't end up in an endless loop
			if($bPreventLoop){
				// at least we tried, skip this error
				$bPreventLoop = false;
				continue;
			}
			// add the original error to the newly constructed error
			$_CONTEXT['errors'][$x]['line'] = !isset($_CONTEXT['errors'][$x]['line']) ? 0 : $_CONTEXT['errors'][$x]['line'];
			$_CONTEXT['errors'][$x]['file'] = !isset($_CONTEXT['errors'][$x]['file']) ? $_PATHS['log_root'].'/log_errors.php' : $_CONTEXT['errors'][$x]['file'];
			$_CONTEXT['errors'][$x] = array('error'=>'Error couldn\'t be processed by log ('.htmlspecialchars($sLastLogFile).'), invalid parameters - '.$_CONTEXT['errors'][$x]['error'],'type'=>'2','line'=>$_CONTEXT['errors'][$x]['line'],'file'=>$_CONTEXT['errors'][$x]['file']);
			$x-=1;
			$bPreventLoop = true;
		}
	}
	// check if we still have data that needs to be written to a log
	if($sWrite!=""){
		WriteF($_PATHS['log_root']."/".$sLastLogFile,$sWrite,(!file_exists($_PATHS['write_dest']) ? "w" : "a"));
		$sWrite = "";
	}
	$_PATHS['write_dest'] = $_PATHS['write_def'];
}
?>