<?php
/* Handles all development actions and returns results as a JSON encoded value

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 09-02-2024
*/
session_cache_limiter('nocache');
header('Expires: '.gmdate('r',0));
header('Content-type: application/json');
$_DYNAMIC_ROOT = "..";
include_once("../header.php");
include_once("../Modules/thc_ss/properties.php");
$aDataR = array();
$aDataR['jresult'] = false;
$aDataR['jerror'] = "Invalid post data";
foreach($_POST as $sKey=>$sValue){
	$_POST[$sKey] = str_replace("\$","\\\$",$_POST[$sKey]);
}
if(isset($_POST['sAbbr']) && isset($_POST['createmod'])){
	/* create new module */
	$sName = trim(strip_tags($_POST['sName']));
	$sAbbr = trim(strip_tags(strtolower($_POST['sAbbr'])));
	$sDescription = trim(htmlentities($_POST['sDescription']));
	$sVersion = trim(strip_tags($_POST['sVersion']));
	$sAuthor = trim(htmlentities($_POST['sAuthor']));
	$sTags = trim(htmlentities($_POST['sTags']));
	$sUrl = trim(strip_tags($_POST['sUrl']));
	$sCatMod = trim(strip_tags($_POST['sModCat']));
	$iScreenFile = isset($_POST['iScreenFile']) && $_POST['iScreenFile']==1 ? 1 : 0;
	$iTimeLimit = isset($_POST['iTimeLimit']) && $_POST['iTimeLimit']==1 ? 1 : 0;
	$iDataFile = isset($_POST['iDataFile']) && $_POST['iDataFile']==1 ? 1 : 0;
	$_CONTEXT['moddir'] = "../Modules/".$sAbbr;
	// validate
	if(strlen($sAbbr)==6){
		if(!preg_match('/^thc_[a-z]{2}$/',$sAbbr)){
			// wrong abbreviation
			$aDataR['jerror'] = "Invalid abbreviation, for example use thc_<i>xx</i>";
		}
		else{
			if(!preg_match('/^(\d+\.){2}(\d+)$/',$sVersion)){
				// wrong version
				$aDataR['jerror'] = "Invalid format, valid versions are i:i:i";
			}
			else{
				if(strlen($sDescription)<10 || strlen($sDescription)>150){
					// exceeded maximum or lower than minimum (description)
					$aDataR['jerror'] = "Description can be up to 150 characters long with a minimum of 10 characters";
				}
				else{
					if(strlen($sAuthor)<4 || strlen($sAuthor)>50){
						// exceeded maximum or lower than minimum (author)
						$aDataR['jerror'] = "Author can be up to 50 characters long with a minimum of 4 characters";
					}
					else{
						if(!isset($_CONTEXT['modcats'][$sCatMod])){
							$aDataR['jerror'] = "Invalid module category";
						}
						else{
							if(strlen($sName)<1 || strlen($sName)>25){
								// exceeded maximum or lower than minimum (module name)
								$aDataR['jerror'] = "Module name can be up to 50 characters long with a minimum of 1 character";
							}
							else{
								$sTags = preg_replace('/\s*,\s*/',',',$sTags);
								if(strlen($sTags)<5 || strlen($sTags)>75){
									// exceeded maximum or lower than minimum (tags)
									$aDataR['jerror'] = "Tags can be up to 75 characters long with a minimum of 5 characters";
								}
								else{
									if($sUrl==""){
										$sUrl = "http://www.hacksuite.com";
									}
									if(!preg_match('/^(https?|ftp):\/\/[^\s\/$.?#].[^\s]*$/',$sUrl)){
										$aDataR['jerror'] = "Invalid http/https/ftp link";
									}
									else{
										$aTags = explode(",",$sTags);
										$aTags = array_filter($aTags,function($sValue){
											return $sValue!=="";
										});
										$aTags = array_values($aTags);
										// ensure the name and abrreviation of the module are there
										$aTags[] = strtolower($sName);
										$aTags[] = strtolower($sAbbr);
										// we want the module abbreviation and name at the front 
										$aReverse = array_reverse($aTags);
										// get rid of keywords that are duplicate
										$aTagFilter = array_unique($aReverse);
										$_CONTEXT['modstaged'] = "../Modules/".$sAbbr;
										if(!is_dir($_CONTEXT['modstaged'])){
											if(!function_exists("GetDirByDir")){
												$sDest = "../Functions/getdirbydir.php";
												include_once($sDest);
											}
											$bPass = true;
											// find setup files from all modules and make sure we haven't used the module name and its abbreviation before
											$aDirs = GetDirByDir("../Modules",0);
											if(in_array($sAbbr,$aDirs)){
												$bPass = false;
												$aDataR['jerror'] = "Module ".$_CONTEXT['moddir']." already exists";
											}
											else{
												for($x=0;$x<count($aDirs);$x++){
													if(substr($aDirs[$x],0,4)=="thc_"){
														include("../Modules/".$aDirs[$x]."/setup.php");
														if(isset($aProperties['name']) && $aProperties['name']==$sName){
															$aDataR['jerror'] = "Name of module ".$_CONTEXT['moddir']." already exists";
															$bPass = false;
															break;
														}
													}
												}
											}
											if($bPass){
												if(false===@mkdir($_CONTEXT['modstaged'],0777)){
													$_CONTEXT['errors'][] = array('error'=>'Unable to create: '.$_CONTEXT['moddir'],'type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
													include_once($_PATHS['end']);
												}
												else{
													if(!function_exists("WriteF")){
														$sDest = "../Functions/fwrite.php";
														include_once($sDest);
													}
													// create setup file
													$_CONTEXT['setuptemplate'] = "<?php\n";
													$_CONTEXT['setuptemplate'] .= "/* properties for module */\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties = array();\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['name'] = \"".$sName."\";\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['version'] = \"".$sVersion."\";\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['author'] = \"".$sAuthor."\";\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['description'] = \"".$sDescription."\";\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['category'] = \"".$sCatMod."\";\n";
													$_CONTEXT['setuptemplate'] .= "\$aProperties['keywords'] = array(";
													for($x=0;$x<count($aTagFilter);$x++){
														if($x!=0){
															$_CONTEXT['setuptemplate'] .= ",";
														}
														$_CONTEXT['setuptemplate'] .= "\"".$aTagFilter[$x]."\"";
													}
													$_CONTEXT['setuptemplate'] .= ");\n";
													$_CONTEXT['setuptemplate'] .= "?>\n";
													// write setup file
													$aDataR['jresult'] = (!WriteF($_CONTEXT['moddir']."/setup.php",$_CONTEXT['setuptemplate'],"w") ? false : true);
													$aDataR['jmodname'] = $sAbbr;
													if($aDataR['jresult']){
														// create index file
														$_CONTEXT['index_template'] = "<?php\n";
														$_CONTEXT['index_template'] .= "/* ".$sName."\n\n";
														$_CONTEXT['index_template'] .= "Author: ".$sAuthor."\n";
														$_CONTEXT['index_template'] .= "Site: ".$sUrl."\n";
														$_CONTEXT['index_template'] .= "Last Edit: ".date($_CONTEXT['time_pattern'],time())."\n";
														$_CONTEXT['index_template'] .= "*/\n";
														$_CONTEXT['index_template'] .= "if(!defined(\"IN_SCRIPT\")){\n";
														$_CONTEXT['index_template'] .= "	exit;\n";
														$_CONTEXT['index_template'] .= "}\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['".$sAbbr."'] = array();\n";
														$_CONTEXT['index_template'] .= "/* keeps track of the current row when defining properties, which makes it easier to add and remove new rows */\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['currentrow'] = 0;\n";
														$_CONTEXT['index_template'] .= "/* form properties */\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['module'] = \$_CONTEXT['module_current'];\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['form'] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['form']['target'] = \$_CONTEXT['screen']['name'];\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['form']['method'] = \"post\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['form']['action'] = \$_CONTEXT['screen']['src'];\n";
														$_CONTEXT['index_template'] .= "/* table */\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table'] = array();\n";
														$_CONTEXT['index_template'] .= "/* table header */\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['header'] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['header']['name'] = \$_CONTEXT['modules'][\$_CONTEXT['mfs']['module']]['name'];\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['header']['description'] = \$_CONTEXT['modules'][\$_CONTEXT['mfs']['module']]['description'];\n";
														$_CONTEXT['index_template'] .= "/* table rows */\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['class'] = \"modrow\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][0] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][0]['class'] = \"modleft\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][0]['value'] = \"test\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][1] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][1]['class'] = \"modright\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['cells'][1]['value'] = \"test\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['currentrow']++;\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']] = array();\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['class'] = \"modrow\";\n";
														$_CONTEXT['index_template'] .= "\$_CONTEXT['mfs']['table']['rows'][\$_CONTEXT['mfs']['currentrow']]['value'] = \"<div class=\\\"modrowleft mt5\\\"><input type=\\\"submit\\\" name=\\\"submit\\\" value=\\\"Test\\\" /></div>\";\n";
														$_CONTEXT['index_template'] .= "// create output form\n";
														$_CONTEXT['index_template'] .= "\$sCode = ModForm(\$_CONTEXT['mfs']);\n";
														$_CONTEXT['index_template'] .= "?>";
														// write index file
														$aDataR['jresult'] = (!WriteF($_CONTEXT['moddir']."/index.php",$_CONTEXT['index_template'],"w") ? false : true);
														if($aDataR['jresult']){
															// screen template
															if($iScreenFile){
																$_CONTEXT['screen_template'] = "<?php\n";
																$_CONTEXT['screen_template'] .= "/*\n";
																$_CONTEXT['screen_template'] .= "Handler for ".$sName."\n\n";
																$_CONTEXT['screen_template'] .= "Author: ".$sAuthor."\n";
																$_CONTEXT['screen_template'] .= "Site: ".$sUrl."\n";
																$_CONTEXT['screen_template'] .= "Last Edit: ".date($_CONTEXT['time_pattern'],time())."\n";
																$_CONTEXT['screen_template'] .= "*/\n";
																if($iTimeLimit){
																	$_CONTEXT['screen_template'] .= "set_time_limit(0);\n";
																}
																$_CONTEXT['screen_template'] .= "\$_MODULE_C = \"".$sAbbr."\";\n";
																$_CONTEXT['screen_template'] .= "\$_CONTEXT['".$sAbbr."'] = array();\n";
																$_CONTEXT['screen_template'] .= "include_once(\"../../Includes/screen_header.php\");\n";
																$_CONTEXT['screen_template'] .= "include_once(\$_PATHS['style_root'].\"/screen.php\");\n";
																$_CONTEXT['screen_template'] .= "include_once(\$_PATHS['functions_root'].\"/scrn.php\");\n";
																$_CONTEXT['screen_template'] .= "// options\n";
																$_CONTEXT['screen_template'] .= "\$sOptions = \"<b>logging:</b> \".(\$_CONTEXT['log']==true ? \"on\" : \"off\").\"<br />\\n\";\n";
																$_CONTEXT['screen_template'] .= "\$sOptions .= \"<b>silence:</b> \".(\$_CONTEXT['silent']==true ? \"on\" : \"off\").\"<br />\\n\";\n";
																$_CONTEXT['screen_template'] .= "\$sOptions .= \"<b>verbose:</b> \".(\$_CONTEXT['verbose']==true ? \"on\" : \"off\").\"\\n\";\n";
																$_CONTEXT['screen_template'] .= "if(isset(\$_POST['submit'])){\n";
																$_CONTEXT['screen_template'] .= "	switch(\$_POST['submit']){\n";
																$_CONTEXT['screen_template'] .= "		/* Test */\n";
																$_CONTEXT['screen_template'] .= "		case\"Test\":\n";
																$_CONTEXT['screen_template'] .= "		die(Screen(\$sOut.\"Test\".\$sEnd,\$_CONTEXT['verbose'],\$_CONTEXT['silent']));\n";
																$_CONTEXT['screen_template'] .= "		break;\n";
																$_CONTEXT['screen_template'] .= "		default:\n";
																$_CONTEXT['screen_template'] .= "		die(Screen(\$sOut.\$sOptions.\$sEnd,\$_CONTEXT['verbose'],\$_CONTEXT['silent']));\n";
																$_CONTEXT['screen_template'] .= "	}\n";
																$_CONTEXT['screen_template'] .= "}\n";
																$_CONTEXT['screen_template'] .= "else{\n";
																$_CONTEXT['screen_template'] .= "	die(Screen(\$sOut.\$sOptions.\$sEnd,\$_CONTEXT['verbose'],\$_CONTEXT['silent']));\n";
																$_CONTEXT['screen_template'] .= "}\n";
																$_CONTEXT['screen_template'] .= "?>";
																$aDataR['jresult'] = (!WriteF($_CONTEXT['moddir']."/screen.php",$_CONTEXT['screen_template'],"w") ? false : true);
																if($aDataR['jresult']){
																	if($iDataFile){
																		// mess it up and you will get accounts.php
																		if(!isset($_POST['sDataFile'])){
																			$_POST['sDataFile'] = "accounts.php";
																		}
																		if(!preg_match('/^[^\\/]+\.[^\\/]+$/',$_POST['sDataFile'])){
																			$_POST['sDataFile'] = "accounts.php";
																		}
																		$aDataR['jresult'] = (!WriteF($_CONTEXT['moddir']."/".$_POST['sDataFile'],"","w") ? false : true);
																		if(!$aDataR['jresult']){
																			$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create data file, only setup, screen and index file were created','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
																			include_once($_PATHS['end']);
																		}
																	}
																}
																else{
																	$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create screen file, only setup and index file were created','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
																	include_once($_PATHS['end']);
																}
															}
														}
														else{
															$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create index file, only setup file was created','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
															include_once($_PATHS['end']);
														}
													}
													else{
														$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create setup file','type'=>'1','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
														include_once($_PATHS['end']);
													}
												}
											}
										}
										else{
											$aDataR['jerror'] = "Module path already exists";
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	else{
		// wrong abbreviation
		$aDataR['jerror'] = "Invalid abbreviation name";
	}
	if($aDataR['jresult']){
		$aDataR['jtemplate'] = $_CONTEXT['index_template'];
	}
}
elseif(isset($_POST['sIndex']) && isset($_POST['sModName'])){
	/* add content to module */
	if(!function_exists("WriteF")){
		$sDest = "../Functions/fwrite.php";
		include_once($sDest);
	}
	$_POST['sModName'] = trim($_POST['sModName']);
	if(!preg_match('/^thc_[a-z]{2}$/',$_POST['sModName'])){
		// wrong module name
		$aDataR['jerror'] = "Invalid module name";
	}
	else{
		$bPass = true;
		$_POST['sIndex'] = trim($_POST['sIndex']);
		$sStart = @substr($_POST['sIndex'],0,5);
		$sEnd = @substr($_POST['sIndex'],strlen($_POST['sIndex'])-2);
		$iPos = @strpos($_POST['sIndex'],"\$sCode");
		$_CONTEXT['moddir'] = "../Modules/".$_POST['sModName'];
		if($sStart=="<?php" && $sEnd=="?>"){
			if($iPos>0){
				$aDataR['jresult'] = (!WriteF($_CONTEXT['moddir']."/index.php",str_replace("\\\$","\$",$_POST['sIndex']),"w") ? false : true);
			}
			else{
				// expecting output in variable $sCode
				$aDataR['jerror'] = "You need to define your output in variable \$sCode";
			}
		}
		else{
			// 'invalid' php syntax
			$aDataR['jerror'] = "Filename must have proper open and close php tags at start and end of file";
		}
		// write index file
		$aDataR['jmoddone'] = ($aDataR['jresult']==true ? "<div class=\"spacingmsg\">".$_CONTEXT['result_headers']['result']."Successfully created module, you can now <a href=\"index.php?module=".$_POST['sModName']."\">try if it works</a>.</div>" : $aDataR['jerror']);
	}
}
elseif(isset($_POST['sRelLoc'])){
	/* create new app */
	$sName = trim(htmlspecialchars($_POST['sName']));
	$sUrl = trim(htmlspecialchars($_POST['sUrl']));
	if($sUrl==""){
		$sUrl = "http://www.hacksuite.com";
	}
	if(!preg_match('/^(https?|ftp):\/\/[^\s\/$.?#].[^\s]*$/',$sUrl)){
		$aDataR['jerror'] = "Invalid http/https/ftp link";
		die(json_encode($aDataR));
	}
	$sRelLoc = ExploitFilter(trim($_POST['sRelLoc']),0,1);
	// remove trailing and leading slashes in $sRelLoc and $sRealLoc
	if(false==($iLength = strlen($sRelLoc))){
		// empty installation location
		$aDataR['jerror'] = "Empty installation location";
		die(json_encode($aDataR));
	}
	$sRelLoc = ($sRelLoc[$iLength-1]=="/" ? substr($sRelLoc,0,-1) : $sRelLoc);
	if(preg_match('/^thc_[a-z]{2}$/',$sRelLoc)){
		// can't install a module as an app
		$aDataR['jerror'] = "Can't install a module as an app";
		die(json_encode($aDataR));
	}
	$sRealLoc = @str_replace("//","/",$_PATHS['root']."/Apps/".$sRelLoc);
	$sVersion = @trim(strip_tags($_POST['sVersion']));
	$sAuthor = @trim(htmlentities($_POST['sAuthor']));
	$sDescription = @trim(htmlentities($_POST['sDescription']));
	$sTags = @preg_replace('/\s*,\s*/',',',$_POST['sTags']);
	if(strlen($sTags)<5 || strlen($sTags)>150){
		// exceeded maximum or lower than minimum (tags)
		$aDataR['jerror'] = "Tags can be up to 150 characters long with a minimum of 5 characters";
		die(json_encode($aDataR));
	}
	$aTags = explode(",",$sTags);
	$aTagsLower = array_map("strtolower",$aTags);
	$aTagsLower[] = strtolower($sName);
	$aTags = array_reverse($aTagsLower);
	$aTags = array_unique($aTags);
	$aTags = array_values($aTags);
	$iName = strlen($sName);
	$bPass = false;
	// validate name field
	if(!$iName || $iName>25){
		// invalid name size
		$aDataR['jerror'] = "Invalid name size";
		die(json_encode($aDataR));
	}
	$bUsed = false;
	// check if app name is already being used
	for($x=0;$x<count($aExtra);$x++){
		if($aExtra[$x][0]==$sName){
			$bUsed = true;
			break;
		}
	}
	if($bUsed){
		// name is already used
		$aDataR['jerror'] = "Name is already used";
		die(json_encode($aDataR));
	}
	// check directory
	if(is_dir($sRealLoc)){
		// directory already exists, is it a reserved directory?
		include_once("../Data/reserved.php");
		$sAppPath = substr($sRealLoc,strlen($_PATHS['root']));
		if(!function_exists("WriteF")){
			$sDest = "../Functions/fwrite.php";
			include_once($sDest);
		}
		if(!in_array($sAppPath,$_CONTEXT['reserved'])){
			// ok it's 'safe' to go on, find setup so the program is recognizable(we don't need it here now though)
			if(!file_exists($sRealLoc."/setup.php")){
				// oops...not there
				$aDataR['jerror'] = "You need to create a setup file in order to launch an app";
				$_CONTEXT['errors'][] = array('error'=>'You need to create a setup file in order to launch an app','type'=>'1','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
				include_once($_PATHS['end']);
			}
			// got the setup file, let's bind the program to the environment
			$sTemplate = "<?php\n";
			$sTemplate .= "/*\n";
			$sTemplate .= "Installed apps\n\n";
			$sTemplate .= "Author: Remco Kouw\n";
			$sTemplate .= "Site: http://www.hacksuite.com\n";
			$sTemplate .= "Last Edit: ".date($_CONTEXT['time_pattern'],time())."\n";
			$sTemplate .= "*/\n";
			$sTemplate .= "if(!defined('IN_SCRIPT')){\n";
			$sTemplate .= "\texit;\n";
			$sTemplate .= "}\n";
			$sTemplate .= "\$aExtra = array();\n";
			if(0!=($iExtra = @count($aExtra))){
				for($x=0;$x<$iExtra;$x++){
					$sTemplate .= "\$aExtra[".$x."] = array(\"".$aExtra[$x][0]."\",\"".$aExtra[$x][1]."\");\n";
				}
			}
			// now add the new item
			$sTemplate .= "\$aExtra[".$iExtra."] = array(\"".$sName."\",\"".$sRelLoc."\");\n";
			$sTemplate .= "?>";
			if(!WriteF("../Data/extra.php",$sTemplate,"w")){
				// update failed
				$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to bind the existing app in data directory (extra.php), you can try manually','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
				include_once($_PATHS['end']);
			}
			// binded successfully
			$aDataR['jresult'] = true;
			$aDataR['jappname'] = $sName;
			$aDataR['japplocation'] = $sRelLoc;
		}
		else{
			// this is a reserved directory, so can't install in here
			$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'This is a reserved directory, so can\'t install in here','type'=>'0','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
			include_once($_PATHS['end']);
		}
	}
	else{
		// directory doesn't exist, let's create the app then
		if(strlen($sAuthor)<4 || strlen($sAuthor)>50){
			// exceeded maximum or lower than minimum (author)
			$aDataR['jerror'] = "Author can be up to 50 characters long with a minimum of 4 characters";
			die(json_encode($aDataR));
		}
		// is version valid?
		if(!preg_match('/^(\d+\.){2}(\d+)$/',$sVersion)){
			// wrong version
			$aDataR['jerror'] = "Invalid format, valid versions are i:i:i";
			die(json_encode($aDataR));
		}
		if(strlen($sDescription)<10 || strlen($sDescription)>150){
			// exceeded maximum or lower than minimum (description)
			$aDataR['jerror'] = "Description can be up to 150 characters long with a minimum of 10 characters";
			die(json_encode($aDataR));
		}
		$_CONTEXT['appstaged'] = $sRealLoc;
		if(false===@mkdir($sRealLoc,0777)){
			$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Unable to create: '.$_CONTEXT['appstaged'],'type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
			include_once($_PATHS['end']);
		}
		if(!function_exists("WriteF")){
			$sDest = "../Functions/fwrite.php";
			include_once($sDest);
		}
		// template
		$_CONTEXT['setuptemplate'] = "<?php\n";
		$_CONTEXT['setuptemplate'] .= "/*\n";
		$_CONTEXT['setuptemplate'] .= "Properties for ".$sName."\n\n";
		$_CONTEXT['setuptemplate'] .= "Author: ".$sAuthor."\n";
		$_CONTEXT['setuptemplate'] .= "Site: ".$sUrl."\n";
		$_CONTEXT['setuptemplate'] .= "Last Edit: ".date($_CONTEXT['time_pattern'],time())."\n";
		$_CONTEXT['setuptemplate'] .= "*/\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties = array();\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['name'] = \"".$sName."\";\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['version'] = \"".$sVersion."\";\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['author'] = \"".$sAuthor."\";\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['location'] = \"".$sRelLoc."\";\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['description'] = \"".$sDescription."\";\n";
		$_CONTEXT['setuptemplate'] .= "\$aProperties['keywords'] = array(";
		for($x=0;$x<count($aTags);$x++){
			if($x!=0){
				$_CONTEXT['setuptemplate'] .= ",";
			}
			$_CONTEXT['setuptemplate'] .= "\"".htmlspecialchars($aTags[$x])."\"";
		}
		$_CONTEXT['setuptemplate'] .= ");\n";
		$_CONTEXT['setuptemplate'] .= "?>\n";
		// write file
		$aDataR['jresult'] = (!WriteF($sRealLoc."/setup.php",$_CONTEXT['setuptemplate'],"w") ? false : true);
		$aDataR['jappname'] = $sName;
		$aDataR['japplocation'] = $sRelLoc;
	}
}
elseif(isset($_POST['sIndexM']) && isset($_POST['sAppName']) && isset($_POST['sAppLocation'])){
	/* add content to app */
	if(!function_exists("WriteF")){
		$sDest = "../Functions/fwrite.php";
		include_once($sDest);
	}	
	$sName = ExploitFilter($_POST['sAppName'],1,0);
	$sRelLoc = ExploitFilter($_POST['sAppLocation'],0,1);
	if(!preg_match('/^thc_[a-z]{2}$/',$_POST['sAppName'])){
		// not a module name
		$_POST['sIndexM'] = trim($_POST['sIndexM']);
		$sStart = @substr($_POST['sIndexM'],0,5);
		$sEnd = @substr($_POST['sIndexM'],strlen($_POST['sIndexM'])-2);
		$_CONTEXT['appdir'] = "../Apps/".$sRelLoc;
		if($sStart=="<?php" && $sEnd=="?>"){
			$aDataR['jresult'] = (!WriteF($_CONTEXT['appdir']."/index.php",str_replace("\\\$","\$",$_POST['sIndexM']),"w") ? false : true);
			if($aDataR['jresult']){
				// template the extra pages
				$sTemplate = "<?php\n";
				$sTemplate .= "/*\n";
				$sTemplate .= "Installed apps\n\n";
				$sTemplate .= "Author: Remco Kouw\n";
				$sTemplate .= "Site: http://www.hacksuite.com\n";
				$sTemplate .= "Last Edit: ".date($_CONTEXT['time_pattern'],time())."\n";
				$sTemplate .= "*/\n";
				$sTemplate .= "if(!defined('IN_SCRIPT')){\n";
				$sTemplate .= "\texit;\n";
				$sTemplate .= "}\n";
				$sTemplate .= "\$aExtra = array();\n";
				if(0!=($iExtra = @count($aExtra))){
					for($x=0;$x<$iExtra;$x++){
						$sTemplate .= "\$aExtra[".$x."] = array(\"".$aExtra[$x][0]."\",\"".$aExtra[$x][1]."\");\n";
					}
				}
				// now add the new item
				$sTemplate .= "\$aExtra[".$iExtra."] = array(\"".$sName."\",\"Apps/".$sRelLoc."\");\n";
				$sTemplate .= "?>";
				if(!WriteF("../Data/extra.php",$sTemplate,"w")){
					// update failed
					$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to bind the existing app in data directory (extra.php), you can try manually','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
					include_once($_PATHS['end']);
				}
				else{
					// binded successfully
					$aDataR['jresult'] = true;
					$aDataR['jappname'] = $sName;
				}
			}
			else{
				$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create index.php file','type'=>'1','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
				include_once($_PATHS['end']);
			}
		}
		else{
			// 'invalid' php syntax
			$aDataR['jerror'] = "Filename must have proper open and close php tags at start and end of file";
		}
		// write index file
		$aDataR['jappdone'] = ($aDataR['jresult']==true ? "Successfully created application, you can now <a href=\"Apps/".$sRelLoc."\">try if it works</a>." : $aDataR['jerror']);
	}
	else{
		// can't be a module name
		$aDataR['jerror'] = "You can't use a module as an application";
	}
}
elseif(isset($_POST['sCallback'])){
	/* create a callback */
	$sLoc = @trim(strip_tags($_POST['sLoc']));
	$iInterval = @intval($_POST['iInterval']);
	$iScan = @intval($_POST['iScan']);
	$iNotify = @intval($_POST['iNotify']);
	$iScanTime = @intval($_POST['iScanSec']);
	// don't you just hate using post variables directly? Uhm I do.. ;)
	$sSearch = @$_POST['sSearch'];
	$sMethod = @$_POST['sMethod'];
	$sEmail = @$_POST['sEmail'];
	$sEmailT = @$_POST['sEmailT'];
	$sLogFile = @$_POST['sLogFile'];
	$iKill = @intval($_POST['iKill']);
	$iRegex = @intval($_POST['iRegex']);
	$sLogTemplate = @$_POST['sLogTemplate'];
	$sCallback = $_POST['sCallback'];
	$sSendVars = $_POST['sSendVars'];
	$sScanUrl = $_POST['sScanUrl'];
	if(!@preg_match('/^[0-9a-zA-z_\-]+\.php$/',$sLoc)){
		// invalid filename for callback function
		$aDataR['jerror'] = "Invalid filename for callback function";
		die(json_encode($aDataR));
	}
	if(@stripos($sCallback,"function Callbck(")===false){
		// missing callback function
		$aDataR['jerror'] = "Missing callback function";
		die(json_encode($aDataR));
	}
	$aUrl = @parse_url($sScanUrl);
	if(!isset($aUrl['host']) || !strlen($aUrl['host'])){
		// no host specified
		$aDataR['jerror'] = "No host specified";
		die(json_encode($aDataR));
	}
	if($sSearch==""){
		// insert a string or pattern to search for
		$aDataR['jerror'] = "Insert a string or pattern to search for";
		die(json_encode($aDataR));
	}
	if($iScan>2){
		// only applies to temporary scans
		if($iInterval!==false){
			if($iInterval<0){
				// interval must be a number above 0
				$aDataR['jerror'] = "Interval must be a number above 0";
				die(json_encode($aDataR));
			}
			elseif($iInterval>$iScanTime){
				// interval must be lower than total scan time
				$aDataR['jerror'] = "Interval must be lower than total scan time";
				die(json_encode($aDataR));
			}
			elseif(!is_int($iInterval)){
				// invalid value for interval
				$aDataR['jerror'] = "Invalid value for interval";
				die(json_encode($aDataR));
			}
		}
	}
	if($iNotify==1){
		// send mail
		if(!@filter_var($sEmail,FILTER_VALIDATE_EMAIL)){
			// invalid email specified
			$aDataR['jerror'] = "Invalid email specified";
			die(json_encode($aDataR));
		}
		else{
			if($sEmailT!=""){
				// email template
				$sDest = "../Modules/thc_ss/EMAILT/".$sEmailT;
				if(@file_exists($sDest)){
					@include_once($sDest);
					if(!function_exists("TemplateEmail")){
						// email template function <b>TemplateEmail</b> doesn't exist
						$_CONTEXT['errors'][] = array('error'=>'Email template function <b>TemplateEmail</b> doesn\'t exist','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
						include_once($_PATHS['end']);
					}
				}
				else{
					// email template doesn't exist
					$_CONTEXT['errors'][] = array('error'=>'Email template doesn\'t exist','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
					include_once($_PATHS['end']);
				}
			}
			else{
				// no email template specified
				$aDataR['jerror'] = "No email template specified";
				die(json_encode($aDataR));
			}
		}
	}
	elseif($iNotify==2){
		if($sLogTemplate!=""){
			// log template
			$sDest = "../Modules/thc_ss/LOGT/".$sLogTemplate;
			if(!@file_exists($sDest)){
				// log template file doesn't exist
				$_CONTEXT['errors'][] = array('error'=>'Log template file doesn\'t exist','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
				include_once($_PATHS['end']);
			}
			// log results to a file
			if($sLogFile!=""){
				// log file
				$sDest = "../thc_ss/LOGS/".$sLogFile;
				if(!@file_exists($sDest)){
					// logfile doesn't exist
					$_CONTEXT['errors'][] = array('error'=>'Logfile doesn\'t exist','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
					include_once($_PATHS['end']);
				}
			}
			else{
				// no logfile specified
				$aDataR['jerror'] = "No logfile specified";
				die(json_encode($aDataR));
			}
		}
		else{
			// no logtemplate specified
			$aDataR['jerror'] = "No logfile specified";
			die(json_encode($aDataR));
		}
	}
	$bContinue = true;
	if(!function_exists("GetFilesByDirectory")){
		include_once("../Functions/getfilebydir.php");
	}
	$aFiles = GetFilesByDirectory("../Modules/thc_ss/CALLBACKS");
	for($x=0;$x<count($aFiles);$x++){
		include_once("../Modules/thc_ss/CALLBACKS/".$aFiles[$x]);
		if($_CBACKD[0]==$_POST['sName'] || $_CBACKD[1]==$_POST['sLoc']){
			$bContinue = false;
			break;
		}
	}
	if(!$bContinue){
		// filename must not exist and callback name must be unique
		$aDataR['jerror'] = "Filename must not exist and callback name must be unique";
		die(json_encode($aDataR));
	}
	// do we use preg searches?
	$bPreg = ($iRegex==1 ? true : false);
	// make sure we have a valid method
	if($sMethod!="post"){
		$sMethod = "get";
	}
	// do we have a callback or will we just use static variables or send nothing
	if(@strpos($sSendVars,"=")>0){
		// add to query
		if($sSendVars[0]=="?"){
			// remove the question mark
			$sSendVars = substr($sSendVars,1);
		}
		if($sSendVars[strlen($sSendVars)-1]=="&"){
			// remove the last ampersand to prevent mixed up urls
			$sSendVars = substr($sSendVars,0,-1);
		}
		$aUrl['query'] = (isset($aUrl['query']) && $aUrl['query']!="" ? $aUrl['query']."&".$sSendVars : $sSendVars);
	}
	// construct callback template
	$sTemplate = "<?php\n";
	$sTemplate .= "if(!defined('IN_SCRIPT')){\n";
	$sTemplate .= "	exit;\n";
	$sTemplate .= "}\n";
	$sTemplate .= "// properties for this callback\n";
	$sTemplate .= "\$_CBACKD = array(\"".$_POST['sName']."\",\"".$_POST['sLoc']."\");\n";
	$sTemplate .= "\$_CBACKV = array();\n";
	$iIndexA = 0;
	include_once("../Modules/thc_ss/properties.php");
	foreach($_POST as $sKey=>$sValue){
		if($sValue!="" && $sKey!="sCallback"){
			// value not empty, which is kewl but do we have this variable in the properties file?
			for($i=0;$i<count($_CONTEXT['thc_ss_indexform']);$i++){
				if(isset($_CONTEXT['thc_ss_indexform'][$i]['var']) && $_CONTEXT['thc_ss_indexform'][$i]['var']==$sKey){
					if(!isset($$sKey)){
						continue;
					}
					$sTemplate .= "\$_CBACKV[".$iIndexA."] = array();\n";
					$sTemplate .= "\$_CBACKV[".$iIndexA."]['value'] = \"".$$sKey."\";\n";
					$sTemplate .= "\$_CBACKV[".$iIndexA."]['row'] = \$_CONTEXT['thc_ss_indexform'][".$i."]['row'];\n";
					$sTemplate .= "\$_CBACKV[".$iIndexA."]['fieldclass'] = \$_CONTEXT['thc_ss_indexform'][".$i."]['fieldclass'];\n";
					$sTemplate .= "\$_CBACKV[".$iIndexA."]['visibility'] = 1;\n";
					$iIndexA++;
					continue;
				}
			}
		}
	}
	$sTemplate .= "if(!isset(\$_GETCBACK)){\n";
	$sTemplate .= $_POST['sCallback']."\n";
	$sTemplate .= "}\n";
	$sTemplate .= "?>";
	include_once("../Functions/fwrite.php");
	$aDataR['jresult'] = (WriteF("../Modules/thc_ss/CALLBACKS/".$_POST['sLoc'],$sTemplate,"w") ? true : false);
	if($aDataR['jresult']){
		$aDataR['jcallback'] = 1;
	}
	else{
		$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create callback','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
		include_once($_PATHS['end']);

	}
}
elseif(isset($_POST['sEditFile']) && isset($_POST['sNewData'])){
	/* edit callback or setup file */
	$sPage = "../Functions/fwrite.php";
	if(!IsThere($sPage)){
		$_CONTEXT['errors'][] = array('error'=>'Can\'t find function file <b>fwrite.php</b>','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
		include_once($_PATHS['end']);
	}
	else{
		@require($sPage);
		// filter out file injections
		$_POST['sEditFile'] = ExploitFilter($_POST['sEditFile'],0,1);
		if(substr($_POST['sEditFile'],0,5)=="Apps/"){
			$_POST['sEditFile'] = "../".$_POST['sEditFile'];
		}
		$aDataR['jresult'] = (WriteF($_POST['sEditFile'],str_replace("\\\$","\$",$_POST['sNewData']),"w") ? true : false);
		$aDataR['jeditdone'] = ($aDataR['jresult'] ? 1 : 0);
		// just in case we fail here, error messages won't be read anyway if result is true
		$aDataR['jerror'] = "Failed to overwrite file";
	}
}
elseif(isset($_POST['sModGlueVar'])){
	/* create modglue documentation */
	$sPage = "../Functions/fwrite.php";
	if(!IsThere($sPage)){
		$_CONTEXT['errors'][] = array('error'=>'Can\'t find function file <b>fwrite.php</b>','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
		include_once($_PATHS['end']);
	}
	else{
		@include_once($sPage);
		$_POST['sModGlueVar'] = ExploitFilter($_POST['sModGlueVar'],1,0);
		$sScreenFile = "../Modules/".$_POST['sModGlueVar']."/screen.php";
		if(!file_exists($sScreenFile)){
			$_CONTEXT['errors'][] = array('error'=>'Failed to find screen file, can\'t create documentation','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
			include_once($_PATHS['end']);
		}
		else{
			// get screen.php data and make a copy of its content where we will remove the comments so we can find modglue variables in the source only  
			$sDataFile = file_get_contents($sScreenFile);
			$sDataFileNoComments = preg_replace('/\/\*[\s\S]*?\*\//','',$sDataFile);
			// get modglue variables
			preg_match_all('/\$_CONTEXT\[\'thc_[a-zA-Z]{2}\'\](?:\[\'[a-zA-Z0-9_]+\'\])+/',$sDataFileNoComments,$aMatches);
			$aMatches[0] = @array_unique($aMatches[0]);
			$aMatches[0] = array_values($aMatches[0]);
			if(false==($iMatches = @count($aMatches[0]))){
				$aDataR['jerror'] = "No modglue variables found in module ".$_POST['sModGlueVar'];
			}
			else{
				// create template for modglue variable listing inside the documentation
				$sTemplate = "ModGlue Variables:\n";
				for($x=0;$x<$iMatches;$x++){
					$sTemplate .= $aMatches[0][$x]."\n";
				}
				$iMethod = preg_match('/ModGlue Variables:/',$sDataFile);
				$sModVar = "\$_CONTEXT['".$_POST['sModGlueVar']."']";
				$aDataFile = explode("\n",$sDataFile);
				$sDataFile = "";
				$bSkip = false;
				$bUpdated = false;
				$bTimeFound = false;
				for($x=0;$x<count($aDataFile);$x++){
					if(!$bUpdated){
						if($bSkip){
							// skip the old modglue variables inside the documentation
							if(substr($aDataFile[$x],0,strlen($sModVar))==$sModVar){
								continue;
							}
							else{
								// no more variables, start buffering data
								$bSkip = false;
								// updated data, now update last edit time
								$bUpdated = true;
							}
						}
						if($iMethod==0){
							// create documentation
							if(substr($aDataFile[$x],0,7)=="Author:"){
								$sDataFile .= $sTemplate."\n";
							}
						}
						else{
							// update documentation
							$sFind = "ModGlue Variables:";
							if(substr($aDataFile[$x],0,strlen($sFind))==$sFind){
								$sDataFile .= $sTemplate;
								$bSkip = true;
								continue;
							}
						}
					}
					if(!$bTimeFound){
						$sFind = "Last Edit:";
						if(substr($aDataFile[$x],0,strlen($sFind))==$sFind){
							$bTimeFound = true;
							$sDataFile .= $sFind." ".date($_CONTEXT['time_pattern'],time())."\n";
							continue;
						}
					}
					$sDataFile .= $aDataFile[$x]."\n";
				}
				$aDataR['jresult'] = (WriteF($sScreenFile,$sDataFile,"w") ? true : false);
				if(!$aDataR['jresult']){
					$_CONTEXT['errors'][count($_CONTEXT['errors'])-1] = array('error'=>'Failed to create documentation','type'=>'2','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
					include_once($_PATHS['end']);
				}
				$aDataR['jmodglue'] = 1;
			}
		}
	}
}
elseif(isset($_POST['sTool'])){
	/* upload an avatar for app or module */
	$sToolFolder = "../";
	$sType = "module";
	$bPass = false;
	// determine what kind of software this is
	if(!preg_match('/^thc_[a-z]{2}$/',$_POST['sTool'])){
		$sType = "app";
		// find filepath of the app
		$sDest = "../Data/extra.php";
		include_once($sDest);
		for($x=0;$x<count($aExtra);$x++){
			if($aExtra[$x][0]==$_POST['sTool']){
				$sToolFolder .= $aExtra[$x][1];
				$bPass = true;
				break;
			}
		}
	}
	else{
		if(!function_exists("GetFilesByDirectory")){
			include_once("../Functions/getfilebydir.php");
		}
		// does module exist or not
		$aDirs = GetDirByDir("../Modules",0);
		if(in_array($_POST['sTool'],$aDirs)){
			$sToolFolder .= "Modules/".$_POST['sTool'];
			$bPass = true;
		}
	}
	if(!$bPass){
		$_CONTEXT['errors'][] = array('error'=>'Software doesn\'t exist','type'=>'0','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
		include_once($_PATHS['end']);
	}
	if(!isset($_FILES['sFile']['error']) || $_FILES['sFile']['error']!==UPLOAD_ERR_OK){
		$_CONTEXT['errors'][] = array('error'=>'Failed to upload file','type'=>'0','fatal'=>'1','json'=>'1','jsonvar'=>'jerror','line'=>__LINE__,'file'=>__FILE__);
	}
	else{
		$sTargetFile = $sToolFolder."/".($sType=="app" ? "avatar" : $_POST['sTool']).".png";
		if(strtolower(pathinfo($_FILES['sFile']['name'],PATHINFO_EXTENSION))!="png"){
			$aDataR['jerror'] = "Upload is not a png file";
		}
		else{
			$aImage = @getimagesize($_FILES['sFile']['tmp_name']);
			if($aImage[0]==200 && $aImage[1]==200){
				if(move_uploaded_file($_FILES['sFile']['tmp_name'],$sTargetFile)){
					$aDataR['jerror'] = "Successfully uploaded avatar";
					$aDataR['jresult'] = true;
				}
				else{
					$aDataR['jerror'] = "Failed to transfer uploaded avatar";
				}
			}
			else{
				$aDataR['jerror'] = "Invalid dimensions, 200px width and height are required";
			}
		}
	}
}
echo json_encode($aDataR);
?>