<?php
/*
Database connection class

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 22-07-2023
*/
class DatabaseConnector{
	private $sHost;
	private $sUser;
	private $sPass;
	// only required when connecting to database defined in $aRequiresDatabaseName
	private $sNameOfDatabase;
	private $iDatabase;
	private $sError;
	private $rConnect;
	private $aDump = array();
	// databases that require a database name for a connection to be made: ConnectMSSQL > MSSQL
	private $aRequiresDatabaseName = array("Oracle","MSSQL","Firebird");
	/* setter methods */
	public function SetConnection($sHost,$sUser,$iDatabase,$sNameOfDatabase){
		$this->sHost = $sHost;
		$this->sUser = $sUser;
		$this->sNameOfDatabase = $sNameOfDatabase;
		$this->iDatabase = $iDatabase;
	}
	public function SetPassword($sPass){
		$this->sPass = $sPass;
	}
	public function SetError($sError){
		$this->sError = $sError;
	}
	/* getter methods */
	public function GetConnection(){
		return $this->rConnect;
	}
	public function GetError(){
		return($this->sError);
	}
	public function GetDump(){
		return($this->aDump);
	}
	public function GetRequired(){
		return($this->aRequiresDatabaseName);
	}
	/* CONNECTIONS THAT CAN BE MADE WITHOUT SPECIFYING A DATABASE NAME */
	public function ConnectMySQL(){
		// inintial database connection
		$this->rConnect = @mysqli_connect($this->sHost,$this->sUser,$this->sPass);
		if(false!=($this->rConnect)){
			// do we want a database name and table name dump
			if($this->iDatabase==1){
				$sQuery = "SHOW DATABASES";
				$rResult = @mysqli_query($this->rConnect,$sQuery);
				if(!$rResult){
					$this->sError = mysqli_error($this->rConnect);
					return false;
				}
				$iDatabase = 0;
				while($aRow = @mysqli_fetch_row($rResult)){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $aRow[0];
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$sTables = "SHOW TABLES FROM ".$this->aDump[$iDatabase]['name'];
					$rTables = @mysqli_query($this->rConnect,$sTables);
					if(!$rTables){
						$this->aDump[$iDatabase]['errors'][] = mysqli_error($this->rConnect);
						return false;
					}
					while($aTable = @mysqli_fetch_row($rTables)){
						$this->aDump[$iDatabase]['tables'][] = $aTable[0];
					}
					mysqli_free_result($rTables);
					$iDatabase++;
				}
				mysqli_free_result($rResult);
				return true;
			}
		}
		else{
			return false;
		}
	}
	public function ConnectPostgreSQL(){
		$sConnect = "host={$this->sHost} user={$this->sUser} password={$this->sPass}";
		$this->rConnect = @pg_connect($sConnect);
		if($this->rConnect) {
			// do we want a database name and table name dump
			if($this->iDatabase==1){
				$sQuery = "SELECT datname FROM pg_database WHERE datistemplate=false";
				$rResult = pg_query($this->rConnect, $sQuery);
				if(!$rResult){
					$this->sError = pg_last_error($this->rConnect);
					return false;
				}
				$iDatabase = 0;
				while($aRow = @pg_fetch_row($rResult)){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $aRow[0];
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$sTables = "SELECT tablename FROM pg_catalog.pg_tables WHERE schemaname='public' AND tablename NOT LIKE 'pg_%' FROM {$this->aDump[$iDatabase]['name']}";
					$rTables = pg_query($this->rConnect, $sTables);
					if(!$rTables){
						$this->aDump[$iDatabase]['errors'][] = pg_last_error($this->rConnect);
						return false;
					}
					while($aTable = @pg_fetch_row($rTables)){
						$this->aDump[$iDatabase]['tables'][] = $aTable[0];
					}
					pg_free_result($rTables);
					$iDatabase++;
				}
				pg_free_result($rResult);
				return true;
			}
		}
		else{
			return false;
		}
	}
	public function ConnectMongoDB(){
		$sConnectUser = urlencode($this->sUser);
		$sConnectPass = urlencode($this->sPass);
		$sConnection = "mongodb://{$sConnectUser}:{$sConnectPass}@{$this->sHost}:27017";
		$cClient = new MongoDB\Client($sConnection);
		try{
			$aDatabases = $cClient->listDatabases();
			// Do we want a database name and table name dump
			if($this->iDatabase==1){
				$iDatabase = 0;
				foreach($aDatabases as $oData){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $oData->getName();
					$this->aDump[$iDatabase]['tables'] = array();
					$aLists = $cClient->selectDatabase($this->aDump[$iDatabase]['name'])->listCollections();
					foreach($aLists as $aList){
						$this->aDump[$iDatabase]['tables'][] = $aList->getName();
					}
					$iDatabase++;
				}
				return true;
			}
		}
		catch(MongoDB\Driver\Exception\Exception $e){
			$this->sError = $e->getMessage();
			return false;
		}
	}
	public function ConnectCouchDB() {
		$sConnectUser = urlencode($this->sUser);
		$sConnectPass = urlencode($this->sPass);
		$sConnection = "http://{$sConnectUser}:{$sConnectPass}@{$this->sHost}:5984";
		$cClient = new \PHPCouch\CouchClient($sConnection);
		try{
			// do we want a database name and table name dump
			if($this->iDatabase==1){
				$aDatabases = $cClient->getAllDatabases();
				$iDatabase = 0;
				foreach($aDatabases as $sDatabaseName) {
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $sDatabaseName;
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$aDocuments = $cClient->useDatabase($this->aDump[$iDatabase]['name'])->getAllDocs();
					foreach($aDocuments->rows as $oDocument){
						$this->aDump[$iDatabase]['tables'][] = $oDocument->id;
					}
					$iDatabase++;
				}
				return true;
			}
		}
		catch(\PHPCouch\CouchException $e){
			$this->sError = $e->getMessage();
			return false;
		}
	}
	/* CONNECTIONS THAT ARE MADE WITH A REQUIRED DATABASE NAME */
	public function ConnectFirebird(){
		$this->rConnect = @ibase_connect($this->sHost.':'.$this->sDBName,$this->sUser,$this->sPass);
		if($this->rConnect){
			// do we want a database name and table name dump
			if($this->iDatabase==1){
				$sQuery = "SELECT RDB\$DATABASE AS DB_NAME FROM RDB\$DATABASE";
				$rResult = @ibase_query($sQuery);
				if (!$rResult) {
					$this->sError = ibase_errmsg();
					return false;
				}
				$iDatabase = 0;
				while($aRow = @ibase_fetch_object($rResult)){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $aRow->DB_NAME;
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$sTables = "SELECT RDB\$RELATION_NAME AS TABLE_NAME FROM RDB\$RELATIONS WHERE RDB\$SYSTEM_FLAG = 0";
					$rTables = @ibase_query($sTables);
					if(!$rTables){
						$this->aDump[$iDatabase]['errors'][] = ibase_errmsg();
						return false;
					}
					while($aTable = @ibase_fetch_object($rTables)){
						$this->aDump[$iDatabase]['tables'][] = $aTable->TABLE_NAME;
					}
					ibase_free_result($rTables);
					$iDatabase++;
				}
				ibase_free_result($rResult);
				return true;
			}
		}
		else{
			return false;
		}
	}
	public function ConnectMSSQL() {
		$aOptions = array(
			"Database" => $this->sNameOfDatabase,
			"UID" => $this->sUser,
			"PWD" => $this->sPass,
			"CharacterSet" => "UTF-8",
		);
		// initialize database connection
		$this->rConnect = @sqlsrv_connect($this->sHost,$aOptions);
		if($this->rConnect){
			// do we want a database name and table name dump
			if($this->iDatabase==1){
				$sQuery = "SELECT name FROM sys.databases WHERE database_id>4";
				$rResult = sqlsrv_query($this->rConnect,$sQuery);
				if(!$rResult) {
					$this->sError = sqlsrv_errors();
					return false;
				}
				$iDatabase = 0;
				while($aRow = sqlsrv_fetch_array($rResult,SQLSRV_FETCH_ASSOC)){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $aRow['name'];
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$sTables = "SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_CATALOG='{$this->sNameOfDatabase}'";
					$rTables = sqlsrv_query($this->rConnect,$sTables);
					if(!$rTables){
						$this->aDump[$iDatabase]['errors'][] = sqlsrv_errors();
						return false;
					}
					while($aTable = @sqlsrv_fetch_array($rTables,SQLSRV_FETCH_ASSOC)){
						$this->aDump[$iDatabase]['tables'][] = $aTable['TABLE_NAME'];
					}
					sqlsrv_free_stmt($rTables);
					$iDatabase++;
				}
				sqlsrv_free_stmt($rResult);
				return true;
			}
		}
		else{
			return false;
		}
	}
	public function ConnectOracle() {
		$sConnect = "(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST={$this->sHost})(PORT=1521))(CONNECT_DATA=(SERVER=DEDICATED)(SERVICE_NAME={$this->sNameOfDatabase})))";
		$this->rConnect = @oci_connect($this->sUser,$this->sPass,$sConnect);
		if($this->rConnect){
			// Do we want a database name and table name dump
			if($this->iDatabase==1){
				$sQuery = "SELECT name FROM v\$database";
				$rResult = @oci_parse($this->rConnect,$sQuery);
				if(!oci_execute($rResult)){
					$this->sError = oci_error($rResult);
					oci_free_statement($rResult);
					return false;
				}
				$iDatabase = 0;
				while($aRow = @oci_fetch_assoc($rResult)){
					$this->aDump[$iDatabase] = array();
					$this->aDump[$iDatabase]['name'] = $aRow['NAME'];
					$this->aDump[$iDatabase]['errors'] = array();
					$this->aDump[$iDatabase]['tables'] = array();
					$sTables = "SELECT TABLE_NAME FROM ALL_TABLES WHERE OWNER='{$this->sUser}'";
					$rTables = oci_parse($this->rConnect,$sTables);
					if(!oci_execute($rTables)) {
						$this->aDump[$iDatabase]['errors'][] = oci_error($rTables);
						oci_free_statement($rTables);
						return false;
					}
					while($aTable = oci_fetch_assoc($rTables)){
						$this->aDump[$iDatabase]['tables'][] = $aTable['TABLE_NAME'];
					}
					oci_free_statement($rTables);
					$iDatabase++;
				}
				oci_free_statement($rResult);
				return true;
			}
		}
		else{
			$this->sError = oci_error();
			return false;
		}
	}
}
// find all supported databases in the connection class
$cMethods = new ReflectionClass("DatabaseConnector");
$aMethods = $cMethods->getMethods();
if(!isset($_CONTEXT['thc_db'])){
	$_CONTEXT['thc_db'] = array();
}
$_CONTEXT['thc_db']['supported'] = array();
foreach($aMethods as $oMethod){
	// all connection methods start with Connect, the rest of the method's name is the database type
	if(substr($oMethod->getName(),0,7)=="Connect"){
		$_CONTEXT['thc_db']['supported'][] = substr($oMethod->getName(),7);
	}
}
?>