<?php
/*
Creates the result graph for the scripts test

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 04-08-2023
*/
$aData = unserialize($_GET['data']);
$aResultAverage = unserialize($_GET['resultavg']);
// define chart dimensions
$iItemCount = count($aData);
// define the minimum and maximum widths and the corresponding number of items
$iMinWidth = 200;
$iMaxWidth = 1200;
$iMinItemCount = 2;
$iMaxItemCount = 18;
// calculate the width using linear interpolation
$iWidth = $iMinWidth+(($iItemCount-$iMinItemCount)/($iMaxItemCount-$iMinItemCount))*($iMaxWidth-$iMinWidth);
// ensure the width is within the allowed range
$iChartWidth = max($iMinWidth,min($iWidth,$iMaxWidth));
$iChartHeight = 400;
$iPadding = 40;
$iBottomSpace = 80;
$iRightSpace = 25;
// calculate bar width and spacing
$iBarWidth = ($iChartWidth-2*$iPadding)/(count($aData)*2);
$iBarSpacing = 30;
// calculate chart range and scale factor
$dMaxTime = max(array_column($aData,'time'));
$dTimeScale = ($dMaxTime>0) ? ($iChartHeight-2*$iPadding-$iBottomSpace)/$dMaxTime : 1;
// create a blank image
$rImage = imagecreatetruecolor($iChartWidth+$iRightSpace,$iChartHeight+$iBottomSpace);
// allocate colors
$iBackgroundColor = imagecolorallocate($rImage,255,255,255);
$iAxisColor = imagecolorallocate($rImage,0,0,0);
$iTimeColorA = imagecolorallocate($rImage,0,0,255);
$iTimeColorB = imagecolorallocate($rImage,255,0,0);
$iTextColor = imagecolorallocate($rImage,0,0,0);
// fill the background
imagefilledrectangle($rImage,0,0,$iChartWidth+$iRightSpace,$iChartHeight+$iBottomSpace,$iBackgroundColor);
// draw the x and y axes
imageline($rImage,$iPadding,$iPadding,$iPadding,$iChartHeight-$iPadding-$iBottomSpace,$iAxisColor);
imageline($rImage,$iPadding,$iChartHeight-$iPadding-$iBottomSpace,$iChartWidth-$iPadding+$iRightSpace,$iChartHeight-$iPadding-$iBottomSpace,$iAxisColor);
// draw "Execution Time" label on the vertical axis
imagestringup($rImage,4,$iPadding-30,($iChartHeight-$iPadding-$iBottomSpace+35)/2,"Execution Time",$iTextColor);
// draw data bars and numbers
$iBarX = $iPadding+$iBarSpacing;
$aScripts = array();
foreach($aData as $aItem){
	if(!in_array($aItem['scriptname'],$aScripts)){
		$aScripts[] = $aItem['scriptname'];
	}
    $iBarHeightTime = $aItem['time']*$dTimeScale;
    $iBarColor = ($aItem['scriptname']===$aScripts[0]) ? $iTimeColorA : $iTimeColorB;
    // draw time bar
    imagefilledrectangle($rImage,$iBarX,$iChartHeight-$iPadding-$iBottomSpace,$iBarX+$iBarWidth,$iChartHeight-$iPadding-$iBarHeightTime-$iBottomSpace,$iBarColor);
    // draw number above bar
    $iTextX = $iBarX+($iBarWidth/2)-10;
    imagestring($rImage,2,$iTextX-12,$iChartHeight-$iPadding-$iBarHeightTime-20-$iBottomSpace,number_format($aItem['time'],6),$iTextColor);
    // move to the next bar
    $iBarX += 2*$iBarSpacing;
}
// draw the legend below the x-axis
$iLegendX = $iPadding;
$iLegendY = $iChartHeight-$iPadding+15-$iBottomSpace;
imagefilledrectangle($rImage,$iLegendX,$iLegendY,$iLegendX+10,$iLegendY+10,$iTimeColorA);
imagestring($rImage,5,$iLegendX+20,$iLegendY-2,$aScripts[0],$iTextColor);
$iLegendY += 20;
imagefilledrectangle($rImage,$iLegendX,$iLegendY,$iLegendX+10,$iLegendY+10,$iTimeColorB);
imagestring($rImage,5,$iLegendX+20,$iLegendY-2,$aScripts[1],$iTextColor);
// draw the result average statistics
$iResultX = 40;
$iResultY = $iChartHeight+$iBottomSpace-140;
imagestring($rImage,4,$iResultX,$iResultY,"Results Average:",$iTextColor);
$iResultY += 20;
$dAverageA = $aResultAverage[0]['total']/$aResultAverage[0]['cycles'];
$dAverageB = $aResultAverage[1]['total']/$aResultAverage[1]['cycles'];
if($dAverageA<$dAverageB){
	$dPercentFaster = number_format((1-$dAverageA/$dAverageB)*100,2);
	$sMessage = $aScripts[0]." is ".$dPercentFaster."% faster than ".$aScripts[1];
}
else{
	$dPercentFaster = number_format((1-$dAverageB/$dAverageA)*100,2);
	$sMessage = $aScripts[1]." is ".$dPercentFaster."% faster than ".$aScripts[0];
}
imagestring($rImage,4,$iResultX,$iResultY,$sMessage,$iTextColor);
// output the image
header('Content-Type: image/png');
imagepng($rImage);
imagedestroy($rImage);
?>